#include <stdint.h>

#include <avr/io.h>
#include <avr/fuse.h>





#define UseCkDiv8 0

#define LED_DDRB DDB4
#define LED_PORTB PB4





#if UseCkDiv8

  #define ClockDivider 8
  
#else

  #define ClockDivider 1
  
#endif  



#define DefaultClockRate (9600 * 1000)


#include <util/delay_basic.h>


#define F_CPU (DefaultClockRate / ClockDivider)
//#include <util/delay.h>





#define SetBits(v, e) (v) |= (e)
#define ResetBits(v, e) (v) &= ~(e)
#define TestBits(e1, e2) ((e1) & (e2))





template <uint8_t CfgBitN, uint8_t OutBitN> class PortBOutBit {

  public:
  
  static uint8_t GetCfgBitN (void) { return CfgBitN; }
  static uint8_t GetCfgBitM (void) { return _BV (GetCfgBitN ()); }

  static uint8_t GetOutBitN (void) { return OutBitN; }
  static uint8_t GetOutBitM (void) { return _BV (GetOutBitN ()); }

  static void Set (void) { SetBits (PORTB, GetOutBitM ()); }
  static void Reset (void) { ResetBits (PORTB, GetOutBitM ()); }
  static void Toggle (void) { PORTB ^= GetOutBitM (); }
  static void SetTo (bool On) { if (On) { Set (); } else { Reset (); } }
  
};





template <uint8_t CfgBitN, uint8_t OutBitN> class LedOnGnd : public PortBOutBit <CfgBitN, OutBitN> {

  public:
  
  static uint8_t GetCfgBitN (void) { return PortBOutBit <CfgBitN, OutBitN>::GetCfgBitN (); }
  static uint8_t GetCfgBitM (void) { return PortBOutBit <CfgBitN, OutBitN>::GetCfgBitM (); }

  static uint8_t GetOutBitN (void) { return PortBOutBit <CfgBitN, OutBitN>::GetOutBitN (); }
  static uint8_t GetOutBitM (void) { return PortBOutBit <CfgBitN, OutBitN>::GetOutBitM (); }

  static void On (void) { PortBOutBit <CfgBitN, OutBitN>::Set (); }
  static void Off (void) { PortBOutBit <CfgBitN, OutBitN>::Reset (); }
  
};




template <uint8_t CfgBitN, uint8_t OutBitN> class LedOnVcc : public PortBOutBit <CfgBitN, OutBitN> {

  public:
  
  static uint8_t GetCfgBitN (void) { return PortBOutBit <CfgBitN, OutBitN>::GetCfgBitN (); }
  static uint8_t GetCfgBitM (void) { return PortBOutBit <CfgBitN, OutBitN>::GetCfgBitM (); }

  static uint8_t GetOutBitN (void) { return PortBOutBit <CfgBitN, OutBitN>::GetOutBitN (); }
  static uint8_t GetOutBitM (void) { return PortBOutBit <CfgBitN, OutBitN>::GetOutBitM (); }

  static void On (void) { PortBOutBit <CfgBitN, OutBitN>::Reset (); }
  static void Off (void) { PortBOutBit <CfgBitN, OutBitN>::Set (); }
  
};




FUSES = {
  (LFUSE_DEFAULT
#if !UseCkDiv8
    | uint8_t (~FUSE_CKDIV8)
#endif
  ),
  (HFUSE_DEFAULT
  )
};





  




int main (void) {

  LedOnGnd <LED_DDRB, LED_PORTB> LedA;

  DDRB = (0
    | LedA.GetCfgBitM ()
  );

  while (1) {

    LedA.On ();

    _delay_loop_2 (100);

    LedA.Off ();

    _delay_loop_2 (200);

  }

  return 0;

}
